/*
Copyright 2002 - 2004 UnitedAdmins
Project originally created by OLO (http://olo.counter-strike.pl)

This file is part of StatsMe.

   StatsMe is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   StatsMe is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with StatsMe; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   A copy of the GNU General Public License is available in a file called COPYING.
   See README for more information.

*/

/* $Id: statsme_rank.cpp,v 1.22 2004/03/18 10:44:55 punisher Exp $ */

#include <extdll.h>			// always
#include <sdk_util.h>		// REG_SVR_COMMAND, etc
#include <meta_api.h>		// Plugin_info, etc
#include <time.h>
#include "statsme.h"	// me

stat_t* sm_rank;
stat_t* sm_rank_last;

// a - begin of list, b - pointer for end of list
void DeleteStats(stat_t**a, stat_t**b)
{
	while ( *a ) {
		stat_t* p = *a;
		*a = (*a)->next;
		delete p;
	}
	*b = NULL;
}

void ResetRanking(void)
{
  DeleteStats(&sm_rank,&sm_rank_last);
  time(&sm.last_reset); //save date of this reset
  LOG_MESSAGE(PLID, "statsme.log reset (manual resetranking).");
  SaveRank();
  player_t* pPlayer;
  for ( int i = 1; i <= gpGlobals->maxClients; ++i )
  {
	pPlayer = GET_PLAYER_POINTER_I(i);
	if (pPlayer->ingame)
	{
		pPlayer->pRank = FindThePositionInRank(pPlayer->name,GETPLAYERAUTHID(pPlayer->pEdict));
	}
  }
}

stat_t* FindThePositionInRank(const char* name, const char* authid)
{
	stat_t* p = sm_rank;
	while ( p ) {

		//buzz 1/4/03
		if (sm_storebyauth->value) {
			if ( !strcmp( authid ,p->authid)) {
				if (strcmp( name ,p->name))  // CHG 33
				  strcpy(p->name , name);
				break;
			}	
		} else {
			if ( !strcmp( name ,p->name) )
				break;
		}
		p = p->next;

	}

	if ( !p ) {// not found? so create new one and place as last
		p = new stat_t;
		memset(p, 0 , sizeof(stat_t) ); // reset all to 0
		strcpy(p->name , name);
		//buzz 1/4/03
		strcpy(p->authid , authid);
		p->id = sm_rank_last ? (sm_rank_last->id + 1) : 1; //set id of position in rank			
		p->prev = sm_rank_last; //and put as last in list
		p->next = NULL;
		if (sm_rank_last)
			sm_rank_last->next = p;
		else
			sm_rank = p;
		sm_rank_last = p;
	}

	return p;
}

// used to set points for rank, save some stats and sort in list
void ScorePlayerInRank(const char* name, stat_t* p, stat_t* a)
{
	p->tks += a->tks;
	p->damage += a->damage;
	p->deaths += a->deaths;
	p->kills += a->kills;
	p->sk += a->sk;
	p->shots += a->shots;
	p->hits += a->hits;
	p->hs += a->hs;
	p->acc = ACCURACY_FORMULA(p->shots,p->hits);
	p->eff = EFFEC_FORMULA(p->kills,p->deaths);
	
	for (int i=0;i<20;i++)
		p->userdefined[i] += a->userdefined[i];

	if ( *sm_rankscore->string ) {
		sm.rank = p;
		p->points = atof(smv_putvars(sm_rankscore->string)); // formula from cvar settings
	}
	else {
		p->points = p->eff + p->acc + ( p->hs / 2 );
		//if ( p->kills < sm_killsforrank->value )
		//	p->points /= 3;
	}

	p->points += 1000;
	//if ( (p->kills >= sm_killsforrank->value) && (strncmp(name,"[No C-D]",8)!=0)    )
	if ( (p->kills < sm_killsforrank->value) || (sm_punishnocd->value=1 && strncmp(name,"[No C-D]",8)==0)    )
			p->points /= 10;
	
	stat_t* d = p->prev; //check if we gain
	while ( d && p->points > d->points  ) {
		d->id++;
		p->id--;
		d = d->prev;
	}
	
	if ( d != p->prev ) {//if we have moved up
		p->prev->next = p->next;
		if ( p->next )
			p->next->prev = p->prev;
		else
			sm_rank_last = p->prev;
		if ( d ) {
			p->prev = d;
			p->next = d->next;
			if ( d->next ) d->next->prev = p;
			d->next = p;
		} else {
			sm_rank->prev = p;
			p->next = sm_rank;
			p->prev = NULL;
			sm_rank = p;
		}
	}


	d = p->next; //check if we have lost the position
	while ( d && p->points < d->points  ) {
		d->id--;
		p->id++;
		d = d->next;
	}

	if ( d != p->next ) {//if we have moved down
		if ( p->prev ) 
			p->prev->next = p->next;
		else
			sm_rank = p->next;
		p->next->prev = p->prev;
		if ( d ) {
			p->prev = d->prev;
			p->next = d;
			if ( d->prev ) d->prev->next = p;
			d->prev = p;
		} else {
			sm_rank_last->next = p;
			p->prev = sm_rank_last;
			p->next = NULL;
			sm_rank_last = p;
		}
	}
}

void ShowTopScore(player_t* pPlayer)
{
	static char *spc = ".........................................";
	int len = sprintf(szBuffor,smv_putvars(sm_rankhead->string));
	sm.rank = sm_rank;
	for (int i = 0 ; sm.rank && i < 10 ; ++i) {
		sm.rank->text = spc + (int)( strlen(sm.rank->name) * 1.6 );
		len += sprintf(&szBuffor[len], smv_putvars(sm_rankbody->string) );
		sm.rank = sm.rank->next;
	}
	UTIL_ShowMOTD(pPlayer->pEdict,szBuffor, smv_putvars(sm_ranktitle->string));
}

void LoadRank()
{
#ifdef ADDON_LAYOUT
	FILE *bfp = fopen(UTIL_Pathname("%s/addons/statsme/%s",sm.mod_name,sm_ranklogfile->string), "rb");

	LOG_MESSAGE(PLID, UTIL_Pathname("Loading statsme rank (file \"%s/addons/statsme/%s\") (game \"%s\") (version \"%d\")" , 
			sm.mod_name,sm_ranklogfile->string, sm.mod_name, VRANKVERSION)  );

#else
	FILE *bfp = fopen(UTIL_Pathname("%s/%s",sm.mod_name,sm_ranklogfile->string), "rb");
#endif
	if (!bfp) {
		LOG_MESSAGE(PLID, "Unable to locate statsme.log file.");
		
		return;
	}

	int ver = 0;
	fread(&ver, 1 , sizeof(int) , bfp);
	if (ver >= 14 ){// logs should be backward compatible from 14 on      VRANKVERSION ){ //proceed only if version of log is the same
		time_t /*when,*/ now;
		time(&now); //get current time
		fread(&sm.last_reset, 1 , sizeof(time_t) , bfp); //read when stats were last cleared
		
		fread( &sm.sm_mode, 1, sizeof(int) * 10, bfp);
		
		
		
		DeleteStats(&sm_rank, &sm_rank_last); //reset stats
		if ( sm_resetrank->value > 0 && sm_resetrank->value && difftime(now, sm.last_reset ) > (sm_resetrank->value * 3600) ) {
			time(&sm.last_reset); //save date of this reset
			LOG_MESSAGE(PLID, "statsme.log reset (sm_resetrank).");
		} else { //we can load rank from file
			int rank_num = 0;
			fread(&rank_num , 1, sizeof(int), bfp);
			stat_t d;
			for(int i = 0; i < rank_num; ++i)	{
				fread(d.name , 1,sizeof(char)*32, bfp);  //32
				fread(d.authid , 1,sizeof(char)*38, bfp); // 70
				fread(&d.id, 1, sizeof(int), bfp); // 74
				fread(&d.tks, 1,sizeof(int), bfp); //78
				fread(&d.damage, 1,sizeof(int), bfp); //82
				fread(&d.deaths, 1,sizeof(int), bfp); //86
				fread(&d.kills, 1,sizeof(int), bfp); //90
				fread(&d.sk, 1,sizeof(int), bfp); // 94
				fread(&d.shots, 1,sizeof(int), bfp); // 98
				fread(&d.hits, 1,sizeof(int), bfp); // 102
				fread(&d.hs, 1,sizeof(int), bfp); // 106
				fread(&d.playtime, 1,sizeof(int), bfp); // 110

				if (ver == 15) {
					fread(&d.userdefined,1,sizeof(int) * 20, bfp); 
				} else {
					memset(&d.userdefined,0,sizeof(d.userdefined));
				}

				ScorePlayerInRank( d.name , FindThePositionInRank( d.name, d.authid ),  &d );
			}

			LOG_MESSAGE(PLID, "Loaded %d records" , rank_num); 
		}
	} else {
		LOG_MESSAGE(PLID, "statsme.log reset (rankversion).");
	}
	fclose(bfp);
	LoadPStatsInfo();
}


void LoadPStatsInfo()
{

	float pKPD = 0, pKPM = 0;
	int pRank=0, pKills=0, pDeaths=0, pPoints=0;
	char* pName;
	char* pID;
	char* Val;
	char buffer[500];

	FILE *bfp = fopen(UTIL_Pathname("%s/stats.dat",sm.mod_name), "rt");
	if (!bfp) return;

	fgets( buffer, 500, bfp );
	
	//while( fscanf( bfp, "%s||%s||%d||%d||%d||%.2f||%.2f||%i||", pName, pID, &pRank, &pKills, &pDeaths, &pKPD, &pKPM, &pPoints ) != EOF)
	while (fgets( buffer, 500, bfp )!=NULL)

	{
	
		pName = strtok( buffer, "%" );
		pID = strtok( NULL, "%" );

		

		if (pName && pID) {


			Val = strtok( NULL, "%");
			if (Val) pRank = atoi(Val);

			Val = strtok( NULL, "%");
			if (Val) pKills = atoi(Val);

			Val = strtok( NULL, "%");
			if (Val) pDeaths = atoi(Val);

			Val = strtok( NULL, "%");
			if (Val) pKPD = atof(Val);

			Val = strtok( NULL, "%");
			if (Val) pKPM = atof(Val);

			Val = strtok( NULL, "%");
			if (Val) pPoints = atoi(Val);

			stat_t* p = sm_rank;
			while ( p ) {

				//buzz 1/4/03
				if (sm_storebyauth->value) {
					if ( !strcmp( pID ,p->authid)) 
						break;	
				} else {
					if ( !strcmp( pName ,p->name) )
						break;
				}
				p = p->next;

			}

			if ( p ) {
				p->pstatsRank = pRank;
				p->pstatsKills = pKills;
				p->pstatsDeaths = pDeaths;
				p->pstatsKPD = pKPD;
				p->pstatsKPM = pKPM;
				p->pstatsPoints = pPoints;

			} 
		}
	}




	fclose(bfp);
}

void SaveRank()
{
#ifdef ADDON_LAYOUT
	FILE *bfp = fopen(UTIL_Pathname("%s/addons/statsme/%s",sm.mod_name,sm_ranklogfile->string), "wb");
	LOG_MESSAGE(PLID, UTIL_Pathname("Saving statsme rank (file \"%s/addons/statsme/%s\") (game \"%s\") (version \"%d\")" , 
			sm.mod_name,sm_ranklogfile->string, sm.mod_name, VRANKVERSION)  );
#else
	FILE *bfp = fopen(UTIL_Pathname("%s/%s",sm.mod_name,sm_ranklogfile->string), "wb");
	LOG_MESSAGE(PLID, UTIL_Pathname("Saving statsme rank (file \"%s/%s\") (game \"%s\") (version \"%d\")" , 
			sm.mod_name,sm_ranklogfile->string, sm.mod_name, VRANKVERSION)  );
#endif
	if (!bfp) {
		
		LOG_MESSAGE(PLID, "Could not save statsme.log");
		return;  //file doesn't exist
	}
	int version = VRANKVERSION;
	fwrite(&version, 1, sizeof(int) , bfp);
	fwrite(&sm.last_reset, 1, sizeof(time_t) , bfp);

	
	fwrite( &sm.sm_mode, 1, sizeof(int) * 10, bfp);

	int rank_num = sm_rank_last ? sm_rank_last->id : 0;
	stat_t* p = sm_rank_last; //go from back and uncount not played nicks!
	while ( p && !p->kills  ) {
		rank_num--;
		p = p->prev;
	}
	fwrite(&rank_num , 1, sizeof(int), bfp);
	for (p = sm_rank; rank_num ; p = p->next) {
		fwrite( p->name , 1, sizeof(char) * 32 , bfp);
		fwrite( p->authid , 1, sizeof(char) * 38 , bfp);
		fwrite( &p->id, 1, sizeof(int), bfp);
		fwrite( &p->tks, 1, sizeof(int), bfp);
		fwrite( &p->damage, 1, sizeof(int), bfp);
		fwrite( &p->deaths, 1, sizeof(int), bfp);
		fwrite( &p->kills, 1, sizeof(int), bfp);
		fwrite( &p->sk, 1, sizeof(int), bfp);
		fwrite( &p->shots, 1, sizeof(int), bfp);
		fwrite( &p->hits, 1, sizeof(int), bfp);
		fwrite( &p->hs, 1, sizeof(int), bfp);
		fwrite( &p->playtime, 1, sizeof(int), bfp);
		fwrite( &p->userdefined,1,sizeof(int) * 20, bfp); 
		rank_num--;
	}
	fclose(bfp);

	LOG_MESSAGE(PLID, "Saved %d records" , rank_num); 
}
